@@ -22,6 +22,7 @@
 
 #include "jobtracker.h"
 #include "jobtrackeradaptor.h"
+#include <akonadi/private/instance_p.h>
 #include "akonadiconsole_debug.h"
 #include <QString>
 #include <QStringList>
@@ -62,7 +63,7 @@ public:
         return id < -1;
     }
 
-    void emitUpdated()
+    void startUpdatedSignalTimer()
     {
         if (!timer.isActive() && !disabled) {
             timer.start();
@@ -70,14 +71,12 @@ public:
     }
 
     QStringList sessions;
-    QHash<QString, int> idToSequence;
-    QHash<int, QString> sequenceToId;
-    QHash<QString, QStringList> jobs;
-    QHash<QString, JobInfo> infoList;
+    QHash<QString, int> nameToId;
+    QHash<int, QVector<int> > childJobs;
+    QHash<int, JobInfo> infoList;
     int lastId;
     QTimer timer;
     bool disabled;
-    QList< QPair<int, int> > unpublishedAdds;
     QList< QPair<int, int> > unpublishedUpdates;
 private:
     JobTracker *const q;
@@ -87,40 +86,10 @@ JobTracker::JobTracker(const char *name, QObject *parent)
     : QObject(parent), d(new Private(this))
 {
     new JobTrackerAdaptor(this);
-    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.akonadiconsole"));
+    const QString suffix = Akonadi::Instance::identifier().isEmpty() ? QString() : QLatin1Char('-') + Akonadi::Instance::identifier();
+    QDBusConnection::sessionBus().registerService(QStringLiteral("org.kde.akonadiconsole") + suffix);
     QDBusConnection::sessionBus().registerObject(QLatin1Char('/') + QLatin1String(name),
             this, QDBusConnection::ExportAdaptors);
-
-#if 0
-    // dummy data for testing
-    d->sessions << "one" << "two" << "three";
-    d->jobs.insert("one", QStringList() << "eins");
-    d->jobs.insert("two", QStringList());
-    d->jobs.insert("three", QStringList());
-
-    // create some fake jobs
-    d->jobs.insert("eins", QStringList() << "sub-eins" << "sub-zwei");
-    d->idToSequence.insert("eins", 0);
-    d->sequenceToId.insert(0, "eins");
-    JobInfo info;
-    info.id = "eins";
-    info.parent = -2;
-    d->infoList.insert("eins", info);
-
-    d->jobs.insert("sub-eins", QStringList());
-    d->idToSequence.insert("sub-eins", 1);
-    d->sequenceToId.insert(1, "sub-eins");
-    info.id = "sub-eins";
-    info.parent = 0;
-    d->infoList.insert("sub-eins", info);
-
-    d->jobs.insert("sub-zwei", QStringList());
-    d->idToSequence.insert("sub-zwei", 2);
-    d->sequenceToId.insert(2, "sub-zwei");
-    info.id = "sub-zwei";
-    info.parent = 0;
-    d->infoList.insert("sub-zwei", info);
-#endif
 }
 
 JobTracker::~JobTracker()
@@ -128,74 +97,75 @@ JobTracker::~JobTracker()
     delete d;
 }
 
-void JobTracker::jobCreated(const QString &session, const QString &job, const QString &parent, const QString &jobType, const QString &debugString)
+void JobTracker::jobCreated(const QString &session, const QString &jobName, const QString &parent, const QString &jobType, const QString &debugString)
 {
-    if (d->disabled || session.isEmpty() || job.isEmpty()) {
+    if (d->disabled || session.isEmpty() || jobName.isEmpty()) {
         return;
     }
 
-    if (!parent.isEmpty() && !d->jobs.contains(parent)) {
-        qCWarning(AKONADICONSOLE_LOG) << "JobTracker: Job arrived before its parent! Fix the library!";
+    int parentId = parent.isEmpty() ? -1 /*for now*/ : idForJob(parent);
+
+    if (!parent.isEmpty() && parentId == -1) {
+        qCWarning(AKONADICONSOLE_LOG) << "JobTracker: Job" << jobName << "arrived before its parent" << parent << " jobType=" << jobType << "! Fix the library!";
         jobCreated(session, parent, QString(), QStringLiteral("dummy job type"), QString());
+        parentId = idForJob(parent);
+        assert(parentId != -1);
     }
+    int sessionId = idForSession(session);
     // check if it's a new session, if so, add it
-    if (d->sessions.isEmpty() || !d->sessions.contains(session)) {
+    if (sessionId == -1) {
+        emit aboutToAdd(d->sessions.count(), -1);
         d->sessions.append(session);
-        d->jobs.insert(session, QStringList());
-        d->unpublishedAdds << QPair<int, int>(d->sessions.count() - 1, -1);
+        emit added();
+        sessionId = idForSession(session);
+    }
+    if (parent.isEmpty()) {
+        parentId = sessionId;
     }
 
     // deal with the job
-    if (d->jobs.contains(job)) {
-        if (d->infoList.value(job).state == JobInfo::Running) {
-            qCDebug(AKONADICONSOLE_LOG) << "Job was already known and still running:" << job << "from" << d->infoList.value(job).timestamp.secsTo(QDateTime::currentDateTime()) << "s ago";
+    const int existingId = idForJob(jobName);
+    if (existingId != -1) {
+        if (d->infoList.value(existingId).state == JobInfo::Running) {
+            qCDebug(AKONADICONSOLE_LOG) << "Job was already known and still running:" << jobName << "from" << d->infoList.value(existingId).timestamp.secsTo(QDateTime::currentDateTime()) << "s ago";
         }
-        // otherwise it just means the pointer got reused... replace old job
+        // otherwise it just means the pointer got reused... insert duplicate
     }
 
-    d->jobs.insert(job, QStringList());
+    assert(parentId != -1);
+    QVector<int> &kids = d->childJobs[parentId];
+    const int pos = kids.size();
+
+    emit aboutToAdd(pos, parentId);
+
+    const int id = d->lastId++;
 
     JobInfo info;
-    info.id = job;
-    if (parent.isEmpty()) {
-        info.parent = idForSession(session);
-    } else {
-        info.parent = idForJob(parent);
-    }
+    info.name = jobName;
+    info.parent = parentId;
     info.state = JobInfo::Initial;
     info.timestamp = QDateTime::currentDateTime();
     info.type = jobType;
     info.debugString = debugString;
-    d->infoList.insert(job, info);
-    const int id = d->lastId++;
-    d->idToSequence.insert(job, id);
-    d->sequenceToId.insert(id, job);
+    d->infoList.insert(id, info);
+    d->nameToId.insert(jobName, id); // this replaces any previous entry for jobName, which is exactly what we want
+    kids << id;
 
-    QString daddy;
-    if (parent.isEmpty()) {
-        daddy = session;
-    } else {
-        daddy = parent;
-    }
-
-    assert(!daddy.isEmpty());
-    QStringList kids = d->jobs[daddy];
-    kids << job;
-    const int pos = d->jobs[daddy].size();
-    d->jobs[daddy] = kids;
-
-    d->unpublishedAdds << QPair<int, int>(pos, info.parent);
-    d->emitUpdated();
+    emit added();
 }
 
-void JobTracker::jobEnded(const QString &job, const QString &error)
+void JobTracker::jobEnded(const QString &jobName, const QString &error)
 {
+    if (d->disabled) {
+        return;
+    }
     // this is called from dbus, so better be defensive
-    if (d->disabled || !d->jobs.contains(job) || !d->infoList.contains(job)) {
+    const int jobId = idForJob(jobName);
+    if (jobId == -1 || !d->infoList.contains(jobId)) {
         return;
     }
 
-    JobInfo info = d->infoList[job];
+    JobInfo &info = d->infoList[jobId];
     if (error.isEmpty()) {
         info.state = JobInfo::Ended;
     } else {
@@ -203,26 +173,28 @@ void JobTracker::jobEnded(const QString &job, const QString &error)
         info.error = error;
     }
     info.endedTimestamp = QDateTime::currentDateTime();
-    d->infoList[job] = info;
 
-    d->unpublishedUpdates << QPair<int, int>(d->jobs[jobForId(info.parent)].size() - 1, info.parent);
-    d->emitUpdated();
+    d->unpublishedUpdates << QPair<int, int>(d->childJobs[info.parent].size() - 1, info.parent);
+    d->startUpdatedSignalTimer();
 }
 
-void JobTracker::jobStarted(const QString &job)
+void JobTracker::jobStarted(const QString &jobName)
 {
+    if (d->disabled) {
+        return;
+    }
     // this is called from dbus, so better be defensive
-    if (d->disabled || !d->jobs.contains(job) || !d->infoList.contains(job)) {
+    const int jobId = idForJob(jobName);
+    if (jobId == -1 || !d->infoList.contains(jobId)) {
         return;
     }
 
-    JobInfo info = d->infoList[job];
+    JobInfo &info = d->infoList[jobId];
     info.state = JobInfo::Running;
     info.startedTimestamp = QDateTime::currentDateTime();
-    d->infoList[job] = info;
 
-    d->unpublishedUpdates << QPair<int, int>(d->jobs[jobForId(info.parent)].size() - 1, info.parent);
-    d->emitUpdated();
+    d->unpublishedUpdates << QPair<int, int>(d->childJobs[info.parent].size() - 1, info.parent);
+    d->startUpdatedSignalTimer();
 }
 
 QStringList JobTracker::sessions() const
@@ -230,47 +202,25 @@ QStringList JobTracker::sessions() const
     return d->sessions;
 }
 
-QList<JobInfo> JobTracker::jobs(int id) const
+QVector<int> JobTracker::childJobs(int parentId) const
 {
-    if (d->isSession(id)) {
-        return jobs(sessionForId(id));
-    }
-    return jobs(jobForId(id));
+    return d->childJobs.value(parentId);
 }
 
-QList<JobInfo> JobTracker::jobs(const QString &parent) const
+int JobTracker::jobCount(int parentId) const
 {
-    assert(d->jobs.contains(parent));
-    const QStringList jobs = d->jobs.value(parent);
-    QList<JobInfo> infoList;
-    Q_FOREACH (const QString &job, jobs) {
-        infoList << d->infoList.value(job);
-    }
-    return infoList;
+    return d->childJobs.value(parentId).count();
 }
 
-QStringList JobTracker::jobNames(int id) const
+int JobTracker::jobIdAt(int childPos, int parentId) const
 {
-    if (d->isSession(id)) {
-        return d->jobs.value(sessionForId(id));
-    }
-    return d->jobs.value(jobForId(id));
+    return d->childJobs.value(parentId).at(childPos);
 }
 
 // only works on jobs
 int JobTracker::idForJob(const QString &job) const
 {
-    assert(d->idToSequence.contains(job));
-    return d->idToSequence.value(job);
-}
-
-QString JobTracker::jobForId(int id) const
-{
-    if (d->isSession(id)) {
-        return sessionForId(id);
-    }
-    assert(d->sequenceToId.contains(id));
-    return d->sequenceToId.value(id);
+    return d->nameToId.value(job, -1);
 }
 
 // To find a session, we take the offset in the list of sessions
@@ -278,7 +228,6 @@ QString JobTracker::jobForId(int id) const
 // way we can discern session ids from job ids and use -1 for invalid
 int JobTracker::idForSession(const QString &session) const
 {
-    assert(d->sessions.contains(session));
     return (d->sessions.indexOf(session) + 2) * -1;
 }
 
@@ -298,32 +247,32 @@ int JobTracker::parentId(int id) const
     if (d->isSession(id)) {
         return -1;
     } else {
-        const QString job = d->sequenceToId.value(id);
-        return d->infoList[job].parent;
+        return d->infoList.value(id).parent;
     }
+}
 
+int JobTracker::rowForJob(int id, int parentId) const
+{
+    const QVector<int> children = childJobs(parentId);
+    // Simple version:
+    //return children.indexOf(id);
+    // But we can do faster since the vector is sorted
+    return std::lower_bound(children.constBegin(), children.constEnd(), id) - children.constBegin();
 }
 
 JobInfo JobTracker::info(int id) const
 {
-    return info(jobForId(id));
+    assert(d->infoList.contains(id));
+    return d->infoList.value(id);
 }
 
-JobInfo JobTracker::info(const QString &job) const
-{
-    assert(d->infoList.contains(job));
-    return d->infoList.value(job);
-}
-
-void JobTracker::triggerReset()
+void JobTracker::clear()
 {
     d->sessions.clear();
-    d->idToSequence.clear();
-    d->sequenceToId.clear();
-    d->jobs.clear();
+    d->nameToId.clear();
+    d->childJobs.clear();
     d->infoList.clear();
-
-    Q_EMIT reset();
+    d->unpublishedUpdates.clear();
 }
 
 void JobTracker::setEnabled(bool on)
@@ -338,9 +287,8 @@ bool JobTracker::isEnabled() const
 
 void JobTracker::signalUpdates()
 {
-    Q_EMIT added(d->unpublishedAdds);
-    Q_EMIT updated(d->unpublishedUpdates);
-    d->unpublishedAdds.clear();
-    d->unpublishedUpdates.clear();
+    if (!d->unpublishedUpdates.isEmpty()) {
+        Q_EMIT updated(d->unpublishedUpdates);
+        d->unpublishedUpdates.clear();
+    }
 }
-
