@@ -32,8 +32,6 @@
 #include <QCommandLineOption>
 #include <QDBusMetaType>
 
-#include <stdlib.h>
-
 int main(int argc, char **argv)
 {
     QCoreApplication::setAttribute(Qt::AA_EnableHighDpiScaling);
@@ -49,7 +47,7 @@ int main(int argc, char **argv)
                          QStringLiteral("0.99"),
                          QStringLiteral("The Management and Debugging Console for Akonadi"),
                          KAboutLicense::GPL,
-                         QStringLiteral("(c) 2006-2016 the Akonadi developer"), QString(),
+                         QStringLiteral("(c) 2006-2018 the Akonadi developer"), QString(),
                          QStringLiteral("https://community.kde.org/KDE_PIM/akonadi"));
     QApplication::setWindowIcon(QIcon::fromTheme(QStringLiteral("akonadi")));
     aboutData.addAuthor(QStringLiteral("Tobias König"), QStringLiteral("Author"), QStringLiteral("tokoe@kde.org"));
@@ -73,8 +71,8 @@ int main(int argc, char **argv)
     if (parser.isSet(QStringLiteral("remote"))) {
         const QString akonadiAddr = QStringLiteral("tcp:host=%1,port=31415").arg(parser.value(QStringLiteral("remote")));
         const QString dbusAddr = QStringLiteral("tcp:host=%1,port=31416").arg(parser.value(QStringLiteral("remote")));
-        setenv("AKONADI_SERVER_ADDRESS", akonadiAddr.toLatin1(), 1);
-        setenv("DBUS_SESSION_BUS_ADDRESS", dbusAddr.toLatin1(), 1);
+        qputenv("AKONADI_SERVER_ADDRESS", akonadiAddr.toLatin1());
+        qputenv("DBUS_SESSION_BUS_ADDRESS", dbusAddr.toLatin1());
     }
 
     InstanceSelector instanceSelector(parser.isSet(QStringLiteral("remote")) ? parser.value(QStringLiteral("remote")) : QString());
